@@ -1,14 +1,15 @@
-from collections import defaultdict
 import json
 import logging
 import pickle
 import time
+from collections import defaultdict
 
 import numpy as np
 import pandas as pd
+
+import torch
 from sklearn import preprocessing
 from sklearn.impute import SimpleImputer
-import torch
 
 from ..models import embed_dnn
 from ..preprocessing import preprocessor
@@ -21,7 +22,7 @@ class BanditPredictor:
 
     def __init__(
         self,
-        experiment_params,
+        feature_config,
         float_feature_order,
         id_feature_order,
         id_feature_str_to_int_map,
@@ -33,7 +34,7 @@ class BanditPredictor:
         model_spec=None,
         dense_features_to_use=["*"],
     ):
-        self.experiment_params = experiment_params
+        self.feature_config = feature_config
         self.float_feature_order = float_feature_order
         self.id_feature_order = id_feature_order
         self.id_feature_str_to_int_map = id_feature_str_to_int_map
@@ -46,7 +47,7 @@ class BanditPredictor:
         self.dense_features_to_use = dense_features_to_use
 
         # the ordered choices that we need to score over.
-        self.choices = []
+        self.choices = feature_config["choices"]
 
         # if using dropout to get prediction uncertainty, how many times to score
         # the same observation
@@ -65,8 +66,11 @@ class BanditPredictor:
     def preprocess_input(self, input, choices):
         # score input choices or all choices by default if none provided.
         # expand the input across all of these choices
-        self.choices = choices or self.experiment_params["choices"]
-        expanded_input = [dict(input, **{"decision": d}) for d in self.choices]
+        self.scored_choices = choices = choices or self.choices
+        expanded_input = [
+            dict(input, **{preprocessor.DECISION_FEATURE_NAME: d})
+            for d in self.scored_choices
+        ]
 
         df = pd.DataFrame(expanded_input)
         float_feature_array = np.empty((len(df), 0))
@@ -85,7 +89,7 @@ class BanditPredictor:
                     logger.warning(
                         f"'{feature_name}' expected in context, but missing."
                     )
-                    if self.experiment_params["features"][feature_name]["type"] == "C":
+                    if self.feature_config["features"][feature_name]["type"] == "C":
                         df[feature_name] = preprocessor.MISSING_CATEGORICAL_CATEGORY
                     else:
                         df[feature_name] = None
@@ -98,9 +102,9 @@ class BanditPredictor:
             float_feature_array = np.append(float_feature_array, values, axis=1)
 
         for feature_name in self.id_feature_order:
-            meta = self.experiment_params["features"][feature_name]
+            meta = self.feature_config["features"][feature_name]
             product_set_id = meta["product_set_id"]
-            product_set_meta = self.experiment_params["product_sets"][product_set_id]
+            product_set_meta = self.feature_config["product_sets"][product_set_id]
 
             if feature_name not in df.columns:
                 # Handle passing missing product set feature
@@ -221,17 +225,34 @@ class BanditPredictor:
                     scores = scores[:, 1:]
 
                 if get_ucb_scores:
+
                     assert (
-                        self.model.use_dropout is True
-                    ), "Can only get UCB scores if model was trained with dropout."
-                    self.model.train()
-                    scores_samples = torch.tensor(
-                        [
-                            self.model.forward(**pytorch_input).numpy()
-                            for i in range(self.num_times_to_score)
-                        ]
-                    )
-                    ucb_scores = np.percentile(scores_samples, q=95, axis=0).tolist()
+                        self.model.use_dropout is True or self.model.is_mdn is True
+                    ), "Can only get UCB scores if model was trained with dropout or MDN."
+
+                    # Currently this assumes you only picked one or the other...
+                    if self.model.use_dropout:
+                        self.model.train()
+                        scores_samples = torch.tensor(
+                            [
+                                self.model.forward(**pytorch_input).numpy()
+                                for i in range(self.num_times_to_score)
+                            ]
+                        )
+
+                        ucb_scores = np.percentile(
+                            scores_samples, q=95, axis=0
+                        ).tolist()
+
+                    else:
+                        batch_size = self.model.batch_size
+                        idx = range(scores_samples.shape[0])
+                        mu_est = [i for i in idx if i // batch_size % 2 == 0]
+                        var_est = [i for i in idx if i // batch_size % 2 == 1]
+                        ucb_scores = (
+                            scores_samples[mu_est] + 1.96 * scores_samples[var_est]
+                        )
+
         elif self.model_type in (
             "linear_bandit",
             "gbdt_bandit",
@@ -250,11 +271,13 @@ class BanditPredictor:
         if not ucb_scores:
             ucb_scores = [0.0 for i in range(len(scores))]
 
-        # sort by scores before returning for visual convenience
-        scores = scores.squeeze()
-        ids = self.choices
+        # convert scores to list of floats and round
+        scores = scores.squeeze().tolist()
+        scores = [scores] if not isinstance(scores, list) else scores
+        scores = [round(score, 4) for score in scores]
 
-        zipped = zip(scores, ids, ucb_scores)
+        # sort by scores before returning for visual convenience
+        zipped = zip(scores, self.scored_choices, ucb_scores)
         sorted_scores = sorted(zipped, reverse=True)
         scores, ids, ucb_scores = zip(*sorted_scores)
 
@@ -291,7 +314,7 @@ class BanditPredictor:
         output = {
             "model_type": self.model_type,
             "model_spec": self.model_spec,
-            "experiment_params": self.experiment_params,
+            "feature_config": self.feature_config,
             "float_feature_order": self.float_feature_order,
             "id_feature_order": self.id_feature_order,
             "id_feature_str_to_int_map": self.id_feature_str_to_int_map,
@@ -388,7 +411,7 @@ class BanditPredictor:
             imputers[feature_name] = imputer
 
         return BanditPredictor(
-            experiment_params=config_dict["experiment_params"],
+            feature_config=config_dict["feature_config"],
             float_feature_order=config_dict["float_feature_order"],
             id_feature_order=config_dict["id_feature_order"],
             id_feature_str_to_int_map=config_dict["id_feature_str_to_int_map"],
